// this DIVERSE DSO calls pfFrameTimeStamp with the value in
// HEV_FRAME_TIME_STAMP.  This value is used in sequence files to determine
// when how to advance the sequence.


// Author: John Kelso, kelso@nist.gov, February, 2005

//      Scientific Applications & Visualization Group
//           SAVG web page: http://math.nist.gov/mcsd/savg/
//      Mathematical & Computational Science Division
//           MCSD web page: http://math.nist.gov/mcsd/
//      National Institute of Standards and Technology
//           NIST web page: http://www.nist.gov/

// "This software was developed at the National Institute of Standards
// and Technology by employees of the Federal Government in the course
// of their official duties. Pursuant to title 17 Section 105 of the
// United States Code this software is not subject to copyright
// protection and is in the public domain.  xxxxxxx is an experimental 
// system.  NIST assumes no responsibility whatsoever for its use by 
// other parties, and makes no guarantees, expressed or implied, about 
// its quality, reliability, or any other characteristic.
// 
// We would appreciate acknowledgement if the software is used."

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class frameTimeStamp : public dtkAugment {
  
public:
  
  frameTimeStamp (dpf *app) ;
  int postConfig (void) ;
  int preFrame (void) ;
  
private:
  
  dpf *app ;
  float timeStampIncr ;
  float timeStamp ;

};

////////////////////////////////////////////////////////////////////////
frameTimeStamp::frameTimeStamp (dpf *app_in) :
  dtkAugment("frameTimeStamp") {
  app = app_in ;
  setDescription("set pfFrameTimeStamp") ;
  
  // dtkAugment starts out invalid.
  validate();
}


////////////////////////////////////////////////////////////////////////
// initial setup
int frameTimeStamp::postConfig(void) {

  if(getenv("HEV_FRAME_TIME_STAMP")) {
    if ((sscanf(getenv("HEV_FRAME_TIME_STAMP"),"%f",&timeStampIncr) != 1) ||
	timeStampIncr<=0.f) {
      dtkMsg.add(DTKMSG_ERROR,"frameTimeStamp: "
		 "HEV_FRAME_TIME_STAMP, \"%s\", is not a positive number, "
		 "removing DSO\n", getenv("HEV_FRAME_TIME_STAMP")) ;
      return dtkAugment::REMOVE_OBJECT ;
    }
    dtkMsg.add(DTKMSG_NOTICE,"frameTimeStamp: "
	       "pfFrameTimeStamp Increment = \"%f\"\n",
	       timeStampIncr) ;
    timeStamp = timeStampIncr ;
  } else {
    dtkMsg.add(DTKMSG_NOTICE,"frameTimeStamp: "
	       "HEV_FRAME_TIME_STAMP not set, removing callback\n",
	       timeStamp) ;
    return dtkAugment::REMOVE_OBJECT ;
  }
  
  return DTK_REMOVE_CALLBACK ; 
}


////////////////////////////////////////////////////////////////////////
// dump the current "frame"
int frameTimeStamp::preFrame (void) {

  pfFrameTimeStamp((double)timeStamp) ;
  timeStamp += timeStampIncr ;
  return DTK_CONTINUE ;
}

/************ DTK C++ dtkAugment loader/unloader functions ***************
 *
 * All DTK dtkAugment DSO files are required to declare these two
 * functions.  These function are called by the loading program to get
 * your C++ objects loaded.
 *
 *************************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager) ;
  return new frameTimeStamp(app) ;
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment ;
  return DTKDSO_UNLOAD_CONTINUE ;
}


